%**** This script computes G&R for a bilayered thin-walled artery
%
%*** G&R-CMM base formulation from <Latorre and Humphrey - APLBE 2018>
%								   (without inflammation)
%
%*** DTA WT model data from <Latorre and Humphrey - BMMB 2018>
%							 (with additional inflammation)
%
% Modified to include only one layer (media), have the same parameters for state o
% and h, no inflammatory burden (xi), pressure can be manually changed
%
% Modified for combined stimulus (Notch + phenomenological)
%
% Simulates Hypertension in murine Infrarenal Abdominal Aorta (based on
% data by Bersi et al. 2017 and parameter values of Latorre & Bersi, also
% included (and homogenized) in Irons et al. 2021)

%** marcos.latorre@yale.edu
%** j.g.m.v.asten@tue.nl

% Common abbreviations in variable names:
% m: smooth muscle
% c: collagen
% e: elastin
% M: media
% A: adventitia
% i: intramural stress
% s: shear stress
% t: circumferential
% z: axial
% d: diagonal (collagen direction)
% 
% e.g. KicM is gain parameter K for the intramural stress stimulus of
% collagen in the media

clear

folder = fileparts(which(mfilename));       % Get name of folder containing G&R framework
addpath(genpath(folder))                    % Add that folder plus all its subfolders to path

method = 7;                                 % select method (i.e., which stimuli are active): 1 only Notch, 2 only phenom stress, 3 only phenom WSS, 4 phenom stress + WSS, 5 Notch + phenom stress, 6 Notch + phenom WSS, 7 Notch + phenom stress + phenom WSS, 8 none: all stimuli at 1

Ds = 0.1;						            % G&R time increment
Tqoc = 10;							        % collagen half-life
numHl = 20;							        % number of Half Lives (Tqo) to consider for integration
Epso = 1.0;							        % relative cardiac output at o
days = 300;						            % total simulation time (days)

% Phenomenological gain parameters
K_stress_sm = 0.473;                           % value figs 3-6; for fig 2: 0.473
K_stress_c = 0.55;                          % value figs 3-6; for fig 2: 0.55
K_wss_sm = 1.41;                              % value figs 3-6; for fig 2: 1.41
K_wss_c = 1.65;                                % value figs 3-6; for fig 2: 1.65 
%
load('DTA_cmm.mat')					        % load values needed by CMM for G&R
% 
% Include external Jagged ligands
Js = 0;                                     % Soluble Jagged ligands
Ji = 0;                                     % Immobilized Jagged ligands
%
%% Geometry
rio  = 0.417; 						        % inner radius at o (s = 0)
roo  = 0.417+0.032;						    % outer radius at o (s = 0)
%
hMo = roo-rio;						        % medial thickness at o (s = 0)
hAo = 0; %roo-rMAo;						    % adventitial thickness at o (s = 0)
%
%% Parameters (Irons 2021)
%
ce  = 114;						            % elastin modulus
Get = 1.96; 						        % circumferential deposition stretch elastin
Gez = 1.73; 						        % axial deposition stretch elastin
Bt  = PAR(4);						        % fraction of circumferential collagen within the adventitia
Bz  = PAR(5);						        % fraction of axial collagen within the adventitia
alpdeg = 30.7;
alp = alpdeg*(pi/180);						% orientation of diagonal collagen wrt axial direction
%
Ge = [1/Get/Gez Get Gez];			        % elastin deposition stretches [r t z] in media and adventitia
%
betaM = [Bz 1-Bz];					        % medial betas [BzM 2*BdM]
betaA = [Bt Bz 1-Bt-Bz];			        % adventitial betas [BtA BzA 2*BdA]
%
% Parameters for energy constitutive equations collagen and SMC (initial values at time zero)
c1m0 = 343;					                % c1t muscle at day 0
c2m0 = 1.23;				                % c2t muscle
c1c0 = 450;					                % c1t collagen
c2c0 = 3.51;					            % c2t collagen
Gm0  = 1.17;					            % circumferential deposition stretch (combined medial collagen and smc)
Gc0  = 1.2;					                % deposition stretch (collagen)
%
% Parameter values after 28 days (these don't change here)
c1m4 = c1m0;        					    % c1t muscle at day 28 (week 4)
c2m4 = c2m0;        					    % c2t muscle
c1c4 = c1c0;        					    % c1t collagen
c2c4 = c2c0;        					    % c2t collagen
Gm4  = Gm0;         					    % circumferential deposition stretch (combined medial collagen and smc)
Gc4  = Gc0;         					    % deposition stretch (collagen)
%
% Collect parameters
PARNEW = [ce, Get, Gez, alp, c1m0, c2m0, c1c0, c2c0, Gm0, Gc0, rio, roo];

%% Turnover parameters
% Mass production and removal parameters (k:removal K: production)
% Removal/survival parameters
Tqom = (1/etaq)*Tqoc;				        % smooth muscle half-life
kmco = [1/Tqom,1/Tqoc,1/Tqoc,1/Tqoc,...	    % ko circ m M, ax c M, diag c M, diag c M
    1/Tqoc,1/Tqoc,1/Tqoc,1/Tqoc];	        % ko circ c A, ax c A, diag c A, diag c A
%
kmc = [kmco; kmco];					        % rate parameters
%
% Gain parameters
%
KicM = K_stress_c;
Kim = K_stress_sm;
KscM = K_wss_c;
Ksm = K_wss_sm;

%% Time and some simulation options
%
domain = numHl*Tqom;				        % total integration domain
invals = round(domain/Ds);		            % Ds-length intervals within integration domain
breaks = invals+1;					        % breaks within integration domain
%
t = (0:Ds:domain)';					        % integration times for past histories
%
q = exp(-t*kmco);					        % survival function
%
SimpsErr = simpsons(q,t(1),t(end),[])./(1./kmco) - ones(size(kmco)); % check ... should go to zero if properly integrated
%
%% Mass fractions and mass densities
phioM = [0.079, 0.326, 0 0.595];                % Infrarenal abdominal aorta, homogenized for 1 layer, Irons 2021
phioA = [emco(4)   emco(5)*betaA];	            % local mass fractions of adventitial [e ct cz 2*cd]
%
rho = 1050;							            % arterial mass density
%
rhooM = phioM*rho;					            % medial [e mt cz 2*cd] densities time 0
rhooA = phioA*rho;					            % advent [e ct cz 2*cd] densities time 0
%
rhoReM = rhooM(1);					            % referential medial elastin density (constant)
rhoReA = rhooA(1);					            % referential advent elastin density (constant)
%
% original homeostatic referential density of smc and collagen
%
rhoRf = ones(breaks,1)*[rhooM([2,3,4,4]),...	% circ m M, ax c M, diag c M, diag c M
    rhooA([2,3,4,4])];		                    % circ c A, ax c A, diag c A, diag c A 
%
mN = (ones(breaks,1)*kmco).*rhoRf;	            % nominal mass production rates
%
Ups = ones(size(rhoRf));			            % stimulus function
mR = mN.*Ups;						            % referential mass production rate
%
ltTauInvM = ones(breaks,1);			            % history of medial circum stretches
ltTauInvA = ones(breaks,1);			            % history of advent circum stretches
lzTauInv  = ones(breaks,1);			            % history of axial stretches
%
lTauInv = [ltTauInvM ltTauInvA lzTauInv];	    % stretches history
%
c1m = c1m0*ones(breaks,1);			            % past history of material properties (these don't change here)
c2m = c2m0*ones(breaks,1);
c1c = c1c0*ones(breaks,1);
c2c = c2c0*ones(breaks,1);
Gm  =  Gm0*ones(breaks,1);
Gc  =  Gc0*ones(breaks,1);
%
emch=emco;
[Pfo,so,Pfh,sh] = PreStr(PAR,parT0,parT4,riio,emco,riih,emch,phioM,PARNEW);	% Find pressure P, axial force f, and stresses at times o and h (only o used here)
%
Po    = Pfo(1);						            % inner pressure at o
sgmIo = so(7);						            % tr(sgm) at o
circstresso = so(1);                            % circumferential stress at zero
fo    = Pfo(2);						            % vessel axial force at o
%
steps = round(days/Ds+1);			            % total G&R steps
%
ltM = 1; ltA = 1;					            % circum stretches at o (wrt to reference configuration, which is at o)
JM  = 1; JA  = 1;					            % Jacobians at o (volume ratio)
%
fctxi = 1/3;						            % factor for inflammation evolution
%
%% initialize solution arrays
%
s    = 0:Ds:days;					            % G&R times
ri   = rio*ones(steps,1);			            % inner radius
hM   = hMo*ones(steps,1);			            % medial thickness
hA   = hAo*ones(steps,1);			            % adventitial thickness
rhoR = ones(steps,1)*[rhooM([2,3,4,4]),...	    % circ m M, ax c M, diag c M, diag c M
    rhooA([2,3,4,4])];	                        % circ c A, ax c A, diag c A, diag c A
Upsi = ones(steps,8);				            % stimulus function
Eps  = Epso*ones(steps,1);			            % relative cardiac output Q/Qo
lz   = ones(steps,1);				            % axial stretch relative to state o
f    = fo*ones(steps,1);			            % axial force
xi   = zeros(steps,1);				            % prescribed inflammatory burden
ltsmc_meank = zeros(steps,1);                   % mean smooth muscle stretch, used for Notch model
Notch = zeros(steps,1);                         % average content of Notch receptor
Delta = zeros(steps,1);                         % average content of Delta ligand
Jagged = zeros(steps,1);                        % average content of Jagged ligand
NICD = zeros(steps,1);                          % average content of NICD
NICD_delta = ones(steps,1);                     % relative difference in NICD content
Ups_Nc = ones(steps,1);                         % Notch stimulus for collagen
Ups_Nsm = ones(steps,1);                        % Notch stimulus for smooth muscle
Ups_c = ones(steps,1);                          % Combined stimulus for collagen
Ups_sm = ones(steps,1);                         % Combined stimulus for smooth muscle
Ups_phenom_stress = ones(steps,2);              % phenomenological stimulus intramural stress
Ups_phenom_wss = ones(steps,2);                 % phenomenological stimulus wall shear stress
Dsigma = zeros(steps,1);                        % intramural stress deviation
Dwss = zeros(steps,1);                          % wall shear stress deviation
sigma_circ = zeros(steps,1);                    % circumferential stress
sigma_ax = zeros(steps,1);                      % axial stress
%
%% prescribe inner pressure & inflammation (not active here)
%
wks4 = 28;							            % 4 weeks = 28 days (only used for inflammatory burden, not active here)
%
SP = 28;
factor = 1.4;
P(s<=SP) = (1 + 1/2*(factor-1)*(1-cos(pi*s(s<=SP)/SP)))*Po;   % Makes a gradual increase in P
P(s> SP) = factor*Po;
%
%** prescribe inflammatory burden (not active here)
%
SX = wks4/4; SX2 = wks4; SX3 = SX2+2*SX;	    % auxiliary times
fctrXi = 1/2;								    % remaining inflammation
%
xi(s<=SX) = 1/2*(1-cos(pi*s(s<=SX)/SX));
xi(s>SX & s<=SX2) = 1;
xi(s>SX2 & s<=SX3) = 1 - fctrXi/2*(1-cos(pi*(s(s>SX2 & s<=SX3)-SX2)/(2*SX)));
xi(s>SX3) = 1 - fctrXi;
xi(:)=0;                                        % inflammation inactive: override values with zero
%

%% Determine reference NICD level at homeostasis
%
ltsmco = mean(Gm);                                                                  % Smooth muscle stretch at o, equal to deposition stretch, assuming long period of no changes before o
[No,Do,Jo,Io] = NotchForGR(hM(1), ltsmco, 0, 0);                                    % Run Notch model to determine initial (reference) NICD (inputs: wall thickness, circumferential cell stretch, external Jagged (must be zero for reference state); output: mean Notch, Delta, Jagged, NICD at end)
NICD_ref = mean(Io(2:end-1));                                                       % Reference NICD level at original homeostatic state o
NICD_delta(1) = NICD_ref/NICD_ref;                                                  % Change in NICD wrt reference
ltsmc_meank(1) = ltsmco;                                                            % Store mean smooth muscle stretch
init_cond = [No,Do,Jo,Io];                                                          % Initial conditions for next timestep (will be overwritten after each timestep

%% COMPUTE GROWTH & REMODELING FOR s > 0
%
% solver options (default algorithm Levenberg-Marquardt):
options = optimoptions(@fsolve,'Display','none','FunctionTolerance',1e-9,'StepTolerance',1e-9,'OptimalityTolerance',1e-9);
%
for k = 2:steps
    %
    s(k) = s(k-1) + Ds;					% current G&R time
    fprintf("**Time step %d \n",k)      % print time step to keep track during simulations
    %
    %** update inflammation dependent properties for hereditary integrals (not relevant here, values will stay constant)
    %
    c1m = [c1m0 + (xi(k)^fctxi)*(c1m4-c1m0)
        c1m(1:end-1)];
    c2m = [c2m0 + (xi(k)^fctxi)*(c2m4-c2m0)
        c2m(1:end-1)];
    c1c = [c1c0 + (xi(k)^fctxi)*(c1c4-c1c0)
        c1c(1:end-1)];
    c2c = [c2c0 + (xi(k)^fctxi)*(c2c4-c2c0)
        c2c(1:end-1)];
    Gm  = [ Gm0 + (xi(k)^fctxi)*(Gm4-Gm0)
        Gm(1:end-1)];
    Gc  = [ Gc0 + (xi(k)^fctxi)*(Gc4-Gc0)
        Gc(1:end-1)];
    %
    %** shift past history arrays so that current state is located first
    qaux = q;
    qaux(2:end,:) = qaux(1:end-1,:);
    kmc(2,:) = kmc(1,:);
    %
    rhoRf(2:end,:) = rhoRf(1:end-1,:);
    mN(2:end,:) = mN(1:end-1,:);
    mR(2:end,:) = mR(1:end-1,:);
    lTauInv(2:end,:) = lTauInv(1:end-1,:);
    Ups(2:end,:) = Ups(1:end-1,:);
    %
    rhoRfCor = zeros(1,size(rhoRf,2));	                % trial referential densities
    i = 0;								                % local iteration
    f(k) = f(k-1);						                % trial axial force
    %
    % Perform local iterations until desired tolerance is attained, i.e. referential densities are found with desired accuracy
    maxit = 20;                                                               % Maximum number of iterations                           
    while (norm(rhoRf(1,:)-rhoRfCor)/norm(rhoRf(1,:)) > 1e-3 && i < maxit)    % Compute and evaluate error, note: during the first iteration, rhoRf(1,:) contains the same values as rhoRf(2,:), so the densities from the previous time step)
        if i == maxit-1
            warning("max number of iterations reached")
        end
        %
        % compute circumferential stretches and axial force
        fprintf("  *iteration %d \n",i)                                                                                                                 % print iteration number to keep track during simulation
        parL = [P(k),lz(k),sgmIo,JM,JA,rio,hMo,hAo,ce,Ge,rhoReM,rhoReA,alp,rho,t(1),t(end),Ds];                                                         % collect parameters
        [ltf,fval,eF] = fsolve(@(ltf) LaplaceCMM(ltf,parL,mR,qaux,kmco,kmc,lTauInv,c1m,c2m,c1c,c2c,Gm,Gc,circstresso),[ltM ltA f(k)],options);   % solve Laplace equations for circumferential stretches and axial force    
        %
        ltM  = ltf(1);									% medial circum stretch
        ltA  = ltf(2);									% advent circum stretch
        f(k) = ltf(3);									% vessel axial force
        %
        ltsmc = ltM*lTauInv(:,1).*Gm;                   % Array of circumferential stretches of smooth muscle cells, deposited at each time tau, with respect to their natural configuration (i.e. F(s)*F^-1(tau)*G)
        ltsmc_mean = mean(ltsmc);                       % Mean smooth muscle cell stretch (used in Notch model)
        ltsmc_meank(k) = ltsmc_mean;                    % Store mean smooth muscle cell stretch
        %
        %* additional variables from converged solution
        %
        lrM = JM/ltM/lz(k);								% medial radial strech
        lrA = JA/ltA/lz(k);								% advent radial strech
        %
        hM(k) = lrM*hMo;								% medial thickness
        hA(k) = lrA*hAo;								% advent thickness
        ri(k) = (ltM*(2*rio+hMo)-hM(k))/2;				% inner radius
        %
        lTauInv(1,:) = [1/ltM	1/ltA	1/lz(k)];		% update current stretches
        %
        %* Intramural and shear stress deviations
        %
        % Select which stress measure is used for phenomenological intramural stress contribution
        % Trace Cauchy stress
        Dsgm = ( P(k)*ri(k)/(hM(k)+hA(k)) + f(k)/(pi*(hM(k)+hA(k))*(2*ri(k)+hM(k)+hA(k))) )/sgmIo-1;
        %
        % Only circumferential stress
%         Dsgm = ( P(k)*ri(k)/(hM(k)+hA(k))  )/circstresso-1;
        %
        Dtau = Eps(k)*rio^3/ri(k)^3-1;                  % Wall shear stress deviations
        %
        % Store stresses and stress deviations at each time point
        sigma_circ(k) = P(k)*ri(k)/(hM(k)+hA(k));
        sigma_ax(k) = f(k)/(pi*(hM(k)+hA(k))*(2*ri(k)+hM(k)+hA(k)));
        Dsigma(k) = Dsgm;                               % intramural stress deviations
        Dwss(k) = Dtau;                                 % wall shear stress deviation
        %
        % STIMULI FOR G&R
        %
        % Call Notch model to obtain NICD
        if method == 1 || method == 5 || method == 6 || method == 7
            [N,D,J,I] = NotchForGR(hM(k), ltsmc_mean, Js, Ji, init_cond);     % Run Notch model (inputs: current wall thickness, circumferential cell stretch, external Jagged & initial conditions, output: mean Notch, Delta, Jagged, NICD at end)
            Notch(k) = mean(N);                                                 % Mean over cells
            Delta(k) = mean(D);
            Jagged(k) = mean(J);
            NICD(k) = mean(I(2:end-1));
            ltsmc_meank(k) = ltsmc_mean;                                        % Store mean smc stretch
            NICD_delta(k) = NICD(k)/NICD_ref;                                   % Change in NICD (normalized to reference homeostatic levels)
            %
            % Obtain Notch stimulus functions
            Ups_Nc(k) = getNotchStimCol(NICD_delta(k));                         % Notch stimulus for collagen
            Ups_Nsm(k) = getNotchStimSm(NICD_delta(k));                         % Notch stimulus for smooth muscle
        end

        % Phenomenological stimuli [collagen, smooth muscle]
        Ups_phenom_stress(k,:) = 1 + [KicM,Kim]*Dsgm;
        Ups_phenom_wss(k,:) = 1 - [KscM,Ksm]*Dtau;

        % Activate/deactive stimuli based on chosen method
        switch method
            % Only Notch
            case 1
                Ups_phenom_stress(k,:) = [1,1];         % set phenomenological stimuli to 1
                Ups_phenom_wss(k,:) = [1,1];
            % Only phenom stress
            case 2
                Ups_phenom_wss(k,:) = [1,1];            % set wall shear stress & Notch stimuli to 1
                Ups_Nc(k) = 1;
                Ups_Nsm(k) = 1;
            % Only phenom WSS
            case 3
                Ups_phenom_stress(k,:) = [1,1];         % set intramural stress & Notch stimuli to 1
                Ups_Nc(k) = 1;
                Ups_Nsm(k) = 1;
            % Phenom stress + WSS
            case 4
                Ups_Nc(k) = 1;                          % set Notch stimuli to 1
                Ups_Nsm(k) = 1;
            % Notch + phenom stress
            case 5
                Ups_phenom_wss(k,:) = [1,1];            % set wall shear stress stimulus to 1
            % Notch + phenom WSS
            case 6
                Ups_phenom_stress(k,:) = [1,1];         % set intramural stress stimulus to 1
            % Notch + phenom stress + WSS
            case 7                                      % all active
            % All stimuli set to 1
            case 8
                Ups_phenom_stress(k,:) = [1,1];         
                Ups_phenom_wss(k,:) = [1,1];            
                Ups_Nc(k) = 1;
                Ups_Nsm(k) = 1;
        end
        % Combine active stimuli in total stimulus functions
        Ups_c(k) = Ups_Nc(k) + Ups_phenom_stress(k,1) + Ups_phenom_wss(k,1) - 2;        % total stimulus function collagen
        Ups_sm(k) = Ups_Nsm(k) + Ups_phenom_stress(k,2) + Ups_phenom_wss(k,2) - 2;      % total stimulus function smc
        Ups(1,:) = [Ups_sm(k), Ups_c(k)*ones(1,3), ones(1,4)];                          % collected stimulus for [smooth muscle, 3x medial collagen (axial, 2x diagonal), 4x adventitial collagen (circ, ax, 2x diag, value of 1 in one-layered vessel)
        %
        kmc(1,:)   = kmco*(1+Dsgm^2);                                                   % degradation parameter
        q(2:end,:) = qaux(2:end,:).*( ones(breaks-1,1) * exp(-Ds*mean(kmc)) );          % survival function
        mR(1,:) = mN(1,:).*Ups(1,:);					                                % mass production before mass update (at current G&R time: first row)
        %
        % update/integrate mass densities and recompute mass production
        %
        rhoRfCor = simpsons(mR.*q,t(1),t(end),[]);		% corrected referential mass densities (trial densities)
        %
        mN(1,:) = kmc(1,:).*rhoRfCor;					% update nominal rates
        mR(1,:) = mN(1,:).*Ups(1,:);					% update mass production
        %
        rhoRf(1,:) = simpsons(mR.*q,t(1),t(end),[]);	% update referential mass densities
        %
        JM = ( rhoReM + sum(rhoRf(1,1:3)) ) / rho;		% update medial [e m cz 2*cd] volume ratios
        JA = ( rhoReA + sum(rhoRf(1,5:7)) ) / rho;		% update advent [e ct cz 2*cd] volume ratios
        %
        % update initial conditions Notch model
        if method == 1 || method == 5 || method == 6 || method == 7
            init_cond = [mean(N), mean(D), mean(J), mean(I(2:end-1))];
        end
        %
        i = i+1;										% update iteration counter
        %
        % converged values needed for plots and subsequent timesteps
        %
        rhoR(k,:) = rhoRf(1,:);				            % referential mass densities
        Upsi(k,:) = Ups(1,:);				            % stimulus functions
    end
end

save("fig2_trace_stress","s","P","ltsmc_meank","sigma_circ","sigma_ax","Dsigma","rhoR","hM")
% save("fig2_circ_stress","s","P","ltsmc_meank","sigma_circ","sigma_ax","Dsigma","rhoR","hM")
