function [N_mean, D_mean, J_mean, I_mean] = NotchForGR_extJagged(thick, stretch, init_cond)

% Notch model for G&R, version May 2022
% Includes potential heterogeneous strain distribution
% Includes external Jagged (soluble & immobilized)
% Includes possibility of accepting initial conditions for Notch protein
% content

%% Import strain and geometry data

scell = 0.002;                              % Size cell
ncells = round(thick/scell);                % Determine number of cells (can change during G&R)
stretchcell = stretch*ones(ncells,1);       % Array with stretches for each cell (here constant)
straincell = 1/2*(stretchcell.^2-1);        % Convert cell stretches to Green-Lagrange strain


%% Notch signaling simulation
% constants
gamma = 0.1;    % [1/h] degradation of Notch (N), Delta (D), Jagged (J)
gammaI = 0.5;   % [1/h] degradation of NICD (I)
kc = 5e-04;     % [1/h] strength cis-inhibition
kt = 2.5e-05;   % [1/h] strengtth transactivation
Npr = 1400;     % [number of proteins] production rate Notch in molec
Dpr = 100;      % [number of proteins] production rate Delta in molec
Jpr = 1600;     % [number of proteins] production rate Delta in molec
I0 = 200;       % [number of proteins] threshold of the Hill function in molec
lN = 2.0;       % Lambda Notch of shifted Hill function [dimensionless]
lD = 0.0;       % Lambda Delta of shifted Hill function [dimensionless]
lJ = 2.0;       % Lambda Jagged of shifted Hill function [dimensionless]
nN = 2.0;       % Hill Notch coefficients [dimensionless]
nD = 2.0;       % Hill Delta coefficients [dimensionless]
nJ = 5.0;       % Hill Jagged coefficients [dimensionless]

nf = 1.0;
lDf = 1.0;      % Lambda Delta of shifted Hill function [dimensionless]
lJf = 1.0;      % Lambda Jagged of shifted Hill function [dimensionless]

Jpr_EC = 4000;

% External Jagged parameters
ktJs = kt;              % [1/h] rate receptor - soluble Jagged binding
ktJi = kt;              % [1/h] rate receptor - immobilized Jagged binding
Js = 0;                 % Constant value of soluble Jagged per cell
Ji = 0;                 % Constant value of immobilized Jagged per cell

% -------------------------------------------------------------------------
% variables
tmin = 0;  % start time
dt = 0.05; % time step (dt)
% ncells = 5; % number of cells
switch nargin
    case 2          % No initial conditions specified: use random, so multiple simulations needed
        ncombs = 25; % number of simulations - take average over 100 simulations
        tmax = 200; % end time
    case 3          % Initial conditions specified, only 1 simulations suffices
        ncombs = 1;
        tmax = 100;
end

% -------------------------------------------------------------------------
% Save number of states per length (multiple simulations)
nS = zeros(ncombs,1);
nSR = zeros(ncombs,1);
nR = zeros(ncombs,1);

% Save average protein levels per cell
N_mean = zeros(ncells,1);
D_mean = zeros(ncells,1);
J_mean = zeros(ncells,1);
I_mean = zeros(ncells,1);
I_end = zeros(ncells,1);

% Mechanosensitivity
A_Notch = -5.79;
A_Jagged = -4.17;
Nmf = exp(A_Notch*straincell(1:end));
Jmf = exp(A_Jagged*straincell(1:end));
Nmf = Nmf';
Jmf = Jmf';

t = zeros((tmax/dt),1);
t(1) = tmin;

% array with protein numbers in each cell at each time point
Notch = zeros((tmax/dt),ncells);
Delta = zeros((tmax/dt),ncells);
Jagged = zeros((tmax/dt),ncells);
NICD = zeros((tmax/dt),ncells);
Jagged_s = Js*ones((tmax/dt),ncells);       % [number of proteins] Soluble external Jagged in each cell
Jagged_i = Ji*ones((tmax/dt),ncells);       % [number of proteins] Immobilized Jagged in each cell

% array with protein numbers in each cell at t = tmax for all
N_combs = zeros(ncombs,ncells);
D_combs = zeros(ncombs,ncells);
J_combs = zeros(ncombs,ncells);
I_combs = zeros(ncombs,ncells);

for s = 1:ncombs

    % Initial conditions
    switch nargin

        case 2      % No initial conditions specified: use random
            % random initial condition for each cell, chosen between a certain
            % range (based on bifurgation plots Boareto)
            % same ranges as Boareto used were chosen
            rng('shuffle') % still random after restarting Matlab
            for c_int = 1:ncells
                % value for N, D, J and I at tmin (initial value)
                Notch(1,c_int) = randi([0, 6000],1,1);
                Delta(1,c_int) = randi([0, 6000],1,1);
                Jagged(1,c_int) = randi([0, 6000],1,1);
                NICD(1,c_int) = randi([0, 600],1,1);
            end

        case 3      % Initial conditions specified: use those (from a previous simulation)
            % Specified as average content in init_cond
            Notch(1,:) = ones(1,ncells)*init_cond(1);
            Delta(1,:) = ones(1,ncells)*init_cond(2);
            Jagged(1,:) = ones(1,ncells)*init_cond(3);
            NICD(1,:) = ones(1,ncells)*init_cond(4);
    end

    i = 0;
    for time = tmin+dt:dt:tmax
        i = i + 1;
        % calculate Ne, De and Je for each cell
        % function Hillfunction = HS(I,I0,n,l)
        % function Neighbors = extp_dt(ncells,Notch,Delta,Jagged,NICD,I0,nf,lDf,lJf,i)
       
        Neighbors = extp_F_dt_oneway(ncells,Notch,Delta,Jagged,NICD,I0,nf,lDf,lJf,i,time,tmin,tmax,dt,Jpr_EC);
        NeD = Neighbors(1,:); % number of Notch molecules in neighboring cells
        NeJ = Neighbors(2,:); % number of Notch molecules in neighboring cells
        De = Neighbors(3,:); % number of Delta molecules in neighboring cells
        Je = Neighbors(4,:); % number of Jagged molecules in neighboring cells

        t(i+1) = time;
        % calculate number of proteins (N,D,J,I) at each time point
        % expicit solution t_new = t_old + differential equations
        % store number of proteins at the next time step in the next row
        Notch(i+1,:) = Notch(i,:) + dt*(Npr*Nmf.*HS(NICD(i,:),I0,nN,lN) - Notch(i,:).*((kc*Delta(i,:) + kt*De) ...
            .*HS(NICD(i,:),I0,nf,lDf) + (kc*Jagged(i,:) + kt*Je).*HS(NICD(i,:),I0,nf,lJf)) - gamma*Notch(i,:)) -ktJs.*Notch(i,:).*Jagged_s(i,:) - ktJi.*Notch(i,:).*Jagged_i(i,:);
        Delta(i+1,:) = Delta(i,:) + dt*(Dpr*HS(NICD(i,:),I0,nD,lD) - Delta(i,:).*(kc*Notch(i,:) + kt*NeD) ...
            .*HS(NICD(i,:),I0,nf,lDf) - gamma*Delta(i,:));
        Jagged(i+1,:) = Jagged(i,:) + dt*(Jpr*Jmf.*HS(NICD(i,:),I0,nJ,lJ) - Jagged(i,:).*(kc*Notch(i,:) + kt*NeJ) ...
            .*HS(NICD(i,:),I0,nf,lJf) - gamma*Jagged(i,:));
        NICD(i+1,:) = NICD(i,:) + dt*(kt*Notch(i,:).*(De.*HS(NICD(i,:),I0,nf,lDf) + Je.*HS(NICD(i,:),I0,nf,lJf)) ...
            - gammaI.*NICD(i,:)) + ktJi.*Notch(i,:).*Jagged_i(i,:);
        for r = 1:ncells
            if Notch(i+1,r) < 0
                Notch(i+1,r) = 0;
            end
            if Delta(i+1,r) < 0
                Delta(i+1,r) = 0;
            end
            if Jagged(i+1,r) < 0
                Jagged(i+1,r) = 0;
            end
            if NICD(i+1,r) < 0
                NICD(i+1,r) = 0;
            end
        end
    end

    % save for each simulation the value of N, D, J and I at time = tmax
    % s is simulation number (s/ncombs)
    N_combs(s,:) = Notch(end,:);
    D_combs(s,:) = Delta(end,:);
    J_combs(s,:) = Jagged(end,:);
    I_combs(s,:) = NICD(end,:);

end % end for s = 1:ncombs

% store average protein levels per cell (averaged over simulations)
N_mean(1:ncells) = mean(N_combs,1);
D_mean(1:ncells) = mean(D_combs,1);
J_mean(1:ncells) = mean(J_combs,1);
I_mean(1:ncells) = mean(I_combs,1);
I_end(1:ncells) = I_combs(end,:);           

% determine cell fate based on average NICD value
cstate = zeros(ncombs,ncells);
for q = 1:size(I_combs,1)
    for w = 1:size(I_combs,2)
        if I_combs(q,w) < 100 % undifferentiated (default Fleur: 145)
            cstate(q,w) = 0;
        elseif I_combs(q,w) > 300 % (default Fleur: 300)
            cstate(q,w) = 1;
        else % differentiated
            cstate(q,w) = 0.5;
        end
    end
end

% calculate number of differentiated and undifferentiated cells for each simulation
for z = 1:ncombs
    v = cstate(z,:);
    nS(z) = length(find(v == 0));
    nR(z) = length(find(v == 1));
    nSR(z) = length(find(v == 0.5));
end

% Calculate the average and std of the number of states for all simulations
mean_nS = mean(nS);
mean_nSR = mean(nSR);
mean_nR = mean(nR);
std_nS = std(nS);
std_nSR = std(nSR);
std_nR = std(nR);
